
	/**************************************************************
	 * A simple 1-D simulation to explain surface-brightness fluctuations
	 * by analogy.  We generate a 1-D array of "stars", then sample
	 * that 1-D array at a range of scales.
	 *
	 * Usage:
	 *         sbf    spacing   binning    seed
	 *
	 *   where
	 * 
	 *           spacing      is the typical spacing between stars
	 *                           in the original 1-D array
	 *
	 *           binning      is the binning factor used to view the
	 *                           array; if binning=10, then group together
	 *                           stars in groups of 10 adjacent cells 
	 *                           before printing output
	 * 
	 *           seed         is a seed for random number generator
	 *
	 *
	 * MWR 4/23/2011
	 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

	/* we place stars into a 1-D array of this size */
#define ARRAY_SIZE 100000
static int star_array[ARRAY_SIZE];
static int binned_array[ARRAY_SIZE];


static int debug = 0;
static char usage[] = "sbf spacing= binning= seed= [debug]";

static int place_stars(int spacing, unsigned long int seed);
static int view_stars(int binning);



int
main
	(
	int argc,
	char *argv[]
	)
{
	int i;
	static int spacing = -1; 
	static int binning = -1;
	/* we do allow an acceptable default seed value */
	static unsigned long int seed = 1;
	
	/* parse arguments */
	for (i = 1; i < argc; i++) {
		if (strncmp(argv[i], "debug", 5) == 0) {
			debug = 1;
		}
		if (strncmp(argv[i], "spacing=", 8) == 0) {
			if (sscanf(argv[i] + 8, "%d", &spacing) != 1) {
				fprintf(stderr, "invalid spacing value %s must be integer \n",
							argv[i] + 8);
				exit(1);
			}
		}
		if (strncmp(argv[i], "binning=", 8) == 0) {
			if (sscanf(argv[i] + 8, "%d", &binning) != 1) {
				fprintf(stderr, "invalid binning value %s must be integer \n",
							argv[i] + 8);
				exit(1);
			}
		}
		if (strncmp(argv[i], "seed=", 5) == 0) {
			if (sscanf(argv[i] + 5, "%ld", &seed) != 1) {
				fprintf(stderr, "invalid seed value %s must be integer \n",
							argv[i] + 5);
				exit(1);
			}
		}
	}
	if (debug > 0) {
		printf(" spacing = %d \n", spacing);
		printf(" binning = %d \n", binning);
		printf(" seed    = %ld \n", seed);
	}
	
	/* check for valid values */
	if ((spacing <= 0) || (binning <= 0) || (seed <= 0)) {
		fprintf(stderr, "invalid arg values, try again\n");
		fprintf(stderr, "%s\n", usage);
		exit(1);
	}

	/* initialize random number generator */
	srandom(seed);
	

	/* do the work */
	/* place stars into the array */
	if (place_stars(spacing, seed) != 0) {
		fprintf(stderr, "sbf: place_stars returns with error \n");
		exit(1);
	}
	/* bin the array, print out count of stars in bins */
	if (view_stars(binning) != 0) {
		fprintf(stderr, "sbf: view_stars returns with error \n");
		exit(1);
	}
	

	exit(0);
}


	/*************************************************************
	 * PROCEDURE: place_stars
	 *
	 * DESCRIPTION: Given a spacing and a seed value,
	 *              generate a series of random numbers
	 *              and use them to place stars into a big
	 *              array with some average spacing, but
	 *              small random variations around it.
	 *
	 * RETURNS:     
	 *              0         if all goes well
	 *              1         if an error occurs
	 *
	 */

static int
place_stars
	(
	int spacing,            /* I: avg spacing between stars */
	unsigned long int seed  /* I: seed for random number generator */
	)
{
	int i;
	int next_space;
	double random_fraction;

	/* initialize array with zeroes */
	for (i = 0; i < ARRAY_SIZE; i++) {
		star_array[i] = 0;
	}
	
	/* now walk through array, placing a star at (roughly) the given interval */
	i = 0;
	while (i < ARRAY_SIZE) {
		
		/* place a star at location "i" */
		star_array[i] = 1;
		if (debug > 0) {
			printf("  placed star at i = %d \n", i);
		}

		/* compute the spacing to next star */
		random_fraction = random() / (double) RAND_MAX;
		next_space = floor(spacing*(random_fraction*2.0));
		i += next_space;

	}

	return(0);

}
	




	/*************************************************************
	 * PROCEDURE: view_stars
	 *
	 * DESCRIPTION: Given a binning value, re-bin the
	 *              array of stars into a smaller array,
	 *              and then print out the number of stars
	 *              in each element of the binned array.
	 *
	 * RETURNS:     
	 *              0         if all goes well
	 *              1         if an error occurs
	 *
	 */

static int
view_stars
	(
	int binning             /* I: bin star_array by this factor */
	)
{
	int i, j;
	int num_bins;
	int start_elem;
	int sum;
	double avg_per_bin;
	double ratio_to_avg, residual_fraction;

	/* initialize binned array with zeroes */
	for (i = 0; i < ARRAY_SIZE; i++) {
		binned_array[i] = 0;
	}
	
	/* 
	 * now walk through star_array, counting the number of stars
	 *    in adjacent sets of "binning" elements.  
	 *    Place that number into the next element of "binned_array[]"
	 */
	num_bins = (ARRAY_SIZE / binning) + 1;
	start_elem = 0;
	for (i = 0; i < num_bins; i++) {
		sum = 0;
		for (j = 0; (j < binning) && ((start_elem + j) < ARRAY_SIZE); j++) {
			sum += star_array[start_elem + j];
		}
		binned_array[i] = sum;
		start_elem += binning;
	}

	/* compute average number of stars in a bin */
	sum = 0;
	for (i = 0; i < num_bins - 1; i++) {
		sum += binned_array[i];
	}
	avg_per_bin = sum / (double) (num_bins - 1);
	printf(" avg_per_bin is %12.6f \n", avg_per_bin);

	/* print out results */
	for (i = 0; i < num_bins - 1; i++) {
		ratio_to_avg = binned_array[i] / avg_per_bin;
		residual_fraction = (binned_array[i] - avg_per_bin) / avg_per_bin;
		printf("  %8d   %8d   %12.8lf  %12.7e \n", 
				i, binned_array[i], ratio_to_avg, residual_fraction);
	}

	return(0);

}
	



